package com.retail_store.order_management.service;

import com.retail_store.order_management.constants.Category;
import com.retail_store.order_management.constants.Color;
import com.retail_store.order_management.constants.Gender;
import com.retail_store.order_management.constants.Size;
import com.retail_store.order_management.model.OrderedProduct;
import com.retail_store.order_management.model.Product;
import com.retail_store.order_management.utils.Validate;

import java.sql.Date;
import java.sql.*;
import java.time.LocalDate;
import java.time.LocalTime;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.logging.Logger;

import static com.retail_store.order_management.utils.Database.getConnection;


public class RetailStoreService extends Validate implements RetailStore, ProductFilter {
    //    private Connection connection;
    public List<Product> products;

    public RetailStoreService(Scanner scanner, Logger logger) {
        this.products = new ArrayList<>();
        this.scanner = scanner;
        this.logger = logger;
    }

    public boolean addProduct() {
        boolean productAdded = false;
        String id = productIdGenerator();
        scanner = new Scanner(System.in);
        logger.info("Enter Product details \n Enter Product Name");
        String name = scanner.next();
        name += scanner.nextLine();
        Category category = getCategory(true, "Enter Product Category");
        Color color = getColor(true, "Select color of the product");
        double price = getPrice(true);
        Gender gender = getGender(true, "Product is ideal for");
        Map<Size, Integer> size = getProductSizes();
        Connection connection = getConnection();
        try {
            PreparedStatement preparedStatement = connection.prepareStatement("insert into products values(?,?,?,?,?,?,?,?,?,?,?)");
            preparedStatement.setString(1, id);
            preparedStatement.setString(2, name);
            preparedStatement.setString(3, category.toString());
            preparedStatement.setString(4, color.toString());
            preparedStatement.setDouble(5, price);
            preparedStatement.setString(6, gender.getGender());
            preparedStatement.setInt(7, size.get(Size.S));
            preparedStatement.setInt(8, size.get(Size.M));
            preparedStatement.setInt(9, size.get(Size.L));
            preparedStatement.setInt(10, size.get(Size.XL));
            preparedStatement.setInt(11, size.get(Size.XXL));
            productAdded = preparedStatement.executeUpdate() > 0;
            if (productAdded) {
                logger.info("Your product with name : " + name + " is added successfully");
            } else {
                logger.info("Your product is not added successfully");
                productAdded = addProduct();
            }
        } catch (SQLException e) {
//            e.printStackTrace();
            logger.warning(e.getMessage());
            logger.info("Your product is not added successfully");
            productAdded = addProduct();
        }
        return productAdded;
    }


    public boolean placeOrder() {
        Connection connection = getConnection();
        try {
            connection.setAutoCommit(false);
            List<OrderedProduct> orderedProductList = new ArrayList<>(createOrder(connection));
            if (orderedProductList.isEmpty()) {
                logger.info("Since your order list is empty, unable to place order\n To make an order please enter 1\n Press any key to exit");
                try {
                    int option = scanner.nextInt();
                    if (option == 1) {
                        return placeOrder();
                    } else {
                        return false;
                    }
                } catch (Exception e) {
                    return false;
                }
            }
            String orderId = orderIdGenerator();
            logger.info("Please enter Customer name");
            String name = scanner.next();
            name += scanner.nextLine();
            logger.info("Please enter Phone number");
            String phone = scanner.next();
            Date date = Date.valueOf(LocalDate.now());
            Time time = Time.valueOf(LocalTime.now());
            connection = getConnection();
            double bill = orderedProductList.stream().map(product -> product.getPrice() * product.getQuantity()).reduce(0.00, (totalBill, productBill) -> totalBill + productBill);
            PreparedStatement preparedStatement = connection.prepareStatement("insert into orders values(?,?,?,?,?,?)");
            preparedStatement.setString(1, orderId);
            preparedStatement.setString(2, name);
            preparedStatement.setString(3, phone);
            preparedStatement.setDate(4, date);
            preparedStatement.setTime(5, time);
            preparedStatement.setDouble(6, bill);
            if (preparedStatement.executeUpdate() > 0) {
                preparedStatement = connection.prepareStatement("insert into ordered_products values (?,?,?,?,?,?,?,?,?)");
                for (OrderedProduct product : orderedProductList) {
                    preparedStatement.setString(1, product.getId());
                    preparedStatement.setString(2, product.getName());
                    preparedStatement.setString(3, product.getCategory().toString());
                    preparedStatement.setString(4, product.getColor().toString());
                    preparedStatement.setDouble(5, product.getPrice());
                    preparedStatement.setString(6, product.getIdealFor().getGender());
                    preparedStatement.setString(7, product.getSize().toString());
                    preparedStatement.setInt(8, product.getQuantity());
                    preparedStatement.setString(9, orderId);
                    preparedStatement.addBatch();
                }
                preparedStatement.executeBatch();
            }
            connection.commit();
            return true;
        } catch (SQLException e) {
            logger.warning(e.getMessage());
            try {
                connection.rollback();
            } catch (SQLException ex) {
                logger.warning(ex.getMessage());
            }
            return true;
        }
    }

    @Override
    public Collection<OrderedProduct> createOrder(Connection connection) {
        List<OrderedProduct> cart = new ArrayList<>();
        boolean orderBoolean = true;
        while (orderBoolean) {
            logger.info("********************GAP********************* " + "\n \t (1) Men  \t \t \t \t (2) Women");
            Gender gender = null;
            gender = getGender(false, "Please select...");
            boolean genderBoolean = true;
            while (genderBoolean) {
                Map<Integer, Category> categoryMenu = getCategoriesByGender(gender);
                logger.info("Select the category\nTo change the gender please enter 0");
                int categoryMenuNumber = scanner.nextInt();
                Category category = categoryMenu.get(categoryMenuNumber);
                if (category == null) {
                    break;
                }
                boolean categoryBoolean = true;
                while (categoryBoolean) {
                    Map<Integer, Product> productsByGenderAndCategory = getProductsByGenderAndCategory(gender, category);
                    logger.info("Select the product\n To change the category please enter 0");
                    int productMenuNumber = scanner.nextInt();
                    Product product = productsByGenderAndCategory.get(productMenuNumber);
                    if (product == null) {
                        break;
                    }
                    if (!addProductToCart(cart, product, connection)) {
                        logger.info("Sorry to say this. Currently this product was out of stock.");
                        continue;
                    } else {
                        logger.info("Your product : " + product.getName() + " is added to your cart.");
                    }
                    boolean menuFlag = true;
                    logger.info("Please enter 1 - to continue the shopping \nPress 2 to place the order");
                    while (menuFlag) {
                        try {
                            int order = scanner.nextInt();
                            switch (order) {
                                case 1:
                                    categoryBoolean = false;
                                    genderBoolean = false;
                                    menuFlag = false;
                                    break;
                                case 2:
                                    logger.info("Order Placed");
                                    categoryBoolean = false;
                                    genderBoolean = false;
                                    menuFlag = false;
                                    orderBoolean = false;
                                    break;
                                default:
                                    logger.info("Please enter valid option");
                            }
                        } catch (Exception e) {
                            logger.info("Please enter valid option");
                        }
                    }
                }
            }
        }
        return cart;
    }

    @Override
    public synchronized boolean addProductToCart(Collection<OrderedProduct> cart, Product product, Connection connection) {
        if (product.outOfStock()) {
            return false;
        }
        logger.info(product.showSizes());
        Size size = getSize(true, "Please Select Size");
        logger.info("Please enter quantity required");
        int quantity = getCount();
        int availableQuantity = product.getSize().get(size);
        if (quantity <= availableQuantity) {
            int remaining = availableQuantity - quantity;
            try {
                PreparedStatement preparedStatement = connection.prepareStatement("update gap.products set " + size.getName() + "=? where id =?");
                preparedStatement.setInt(1, remaining);
                preparedStatement.setString(2, product.getId());
                preparedStatement.executeUpdate();
            } catch (SQLException e) {
                logger.info(e.getMessage());
                logger.warning("Unable to add product to cart");
                return addProductToCart(cart, product, connection);
            }
            return cart.add(new OrderedProduct(product.getId(), product.getName(), product.getCategory(), product.getColor(), product.getPrice(), product.getIdealFor(), size, quantity));
        } else {
            logger.warning("your requirement is out of our quantity.\n please enter quantity less or equal to the stock available");
            return addProductToCart(cart, product, connection);
        }
    }

    @Override
    public boolean addProductToCart(Collection<OrderedProduct> cart, Product product, PreparedStatement preparedStatement) {
        return false;
    }


    @Override
    public Map<Integer, Category> getCategoriesByGender(Gender gender) {
        Map<Integer, Category> categoryMap = new LinkedHashMap<>();
        AtomicInteger number = new AtomicInteger(0);
        Connection connection = getConnection();
        try {
            PreparedStatement preparedStatement = connection.prepareStatement("select category from gap.products where gender=? group by category");
            preparedStatement.setString(1, gender.getGender());
            ResultSet resultSet = preparedStatement.executeQuery();
            while (resultSet.next()) {
                categoryMap.put(number.incrementAndGet(), Category.valueOf(resultSet.getString(1)));
            }
        } catch (SQLException e) {
            logger.info(e.getMessage());
            logger.info("got error while fetching the gender. please try again");
            return getCategoriesByGender(gender);
        }
        if (categoryMap.isEmpty()) {
            logger.info("Categories not found in " + gender);
        } else {
            String menu = categoryMap.entrySet().stream().map(entry -> entry.getKey() + ". " + entry.getValue()).reduce("", (categories, category) -> categories + "\n" + category);
            logger.info(menu);
        }
        return categoryMap;
    }

    @Override
    public Map<Integer, Color> getColorsByGenderAndCategory(Gender gender, Category category) {
        Map<Integer, Color> colorMap = new LinkedHashMap<>();
        AtomicInteger number = new AtomicInteger(0);
        Connection connection = getConnection();
        try {
            PreparedStatement preparedStatement = connection.prepareStatement("select color from gap.products where gender=? and category=? group by color");
            preparedStatement.setString(1, gender.getGender());
            preparedStatement.setString(2, category.toString());
            ResultSet resultSet = preparedStatement.executeQuery();
            while (resultSet.next()) {
                colorMap.put(number.incrementAndGet(), Color.valueOf(resultSet.getString(1)));
            }
        } catch (SQLException e) {
            logger.info(e.getMessage());
            logger.info("got error while fetching the gender. please try again");
            return getColorsByGenderAndCategory(gender, category);
        }
        if (colorMap.isEmpty()) {
            logger.info("Colors not found in " + category + " for " + gender);
        } else {
            String menu = colorMap.entrySet().stream().map(entry -> entry.getKey() + ". " + entry.getValue()).reduce("", (colors, color) -> colors + "\n" + color);
            logger.info(menu);
        }
        return colorMap;
    }

    @Override
    public Map<Integer, Product> getProductsByGenderAndCategory(Gender gender, Category category) {
        Map<Integer, Product> productMap = new LinkedHashMap<>();
        AtomicInteger number = new AtomicInteger(0);
        Connection connection = getConnection();
        try {
            PreparedStatement preparedStatement = connection.prepareStatement("select * from gap.products where gender=? and category =?");
            preparedStatement.setString(1, gender.getGender());
            preparedStatement.setString(2, category.toString());
            ResultSet resultSet = preparedStatement.executeQuery();
            ResultSetMetaData resultSetMetaData = resultSet.getMetaData();
            while (resultSet.next()) {
                Product product = new Product(resultSet.getString(1), resultSet.getString(2), Category.valueOf(resultSet.getString(3)), Color.valueOf(resultSet.getString(4)), resultSet.getDouble(5), Gender.valueOf(resultSet.getString(6).toUpperCase()), getProductSizes(resultSet.getInt(7), resultSet.getInt(8), resultSet.getInt(9), resultSet.getInt(10), resultSet.getInt(11)));
                productMap.put(number.incrementAndGet(), product);
            }
        } catch (SQLException e) {
            logger.info(e.getMessage());
            logger.info("got error while fetching the gender. please try again");
            return getProductsByGenderAndCategory(gender, category);
        }
        if (productMap.isEmpty()) {
            logger.info("Products not found in " + category);
        } else {
            String menu = productMap.entrySet().stream().map(entry -> entry.getKey() + ". " + entry.getValue().productShowcaseInMenu()).reduce("", (products, product) -> products + "\n" + product);
            logger.info(menu);
        }
        return productMap;
    }

    @Override
    public Map<Integer, Product> getProductsByGenderAndColor(Collection<Product> productList, Color color) {
        return null;
    }
}
